#ifndef GrObject_H
#define GrObject_H
// $Header: /p/graphics/CVS/GrTown2005/GrObject.H,v 1.4 2005/11/11 03:07:16 gleicher Exp $
// GraphicsTown - CS559-2000, sample solution
// (c) 2000 Michael L. Gleicher
//
// This example code should give you some idea as to how my system
// worked.


#include "MMatrix.H"

// class GrObject: a drawable object in the word of GraphicsTown
// this is the guts
// Objects can sometimes serve as cameras

// other classes referred to:
class DrawingState;
class Behavior;

extern int noNameCtr;
class GrObject {
 public:
  GrObject(char* nameBase = "NoName", int& nameCtr = noNameCtr, char* xn=0);
  virtual ~GrObject();

  // things to know about the object
  bool ridable;					/* can we "ride" this thing */
  bool show_shadow;
  char* name;
  float rideAbove;				// in case we don't want to be stuck on the ground

  // simulation: every object is required to "update itself" to the
  // current time. no guaruntees that this clock is updated
  // uniformly (since it depends on drawing rate). The units are
  // number of milliseconds since the beginning of the simulation
  virtual void simulateUntil(unsigned long time);

  // what makes the object go - by default, this is all that
  // SimulateUntil does.
  // note - this is a linked list! All behaviors get their chance
  Behavior* behaviors;

  // the main thing an object does is draw itself
  // this happens in 2 phases, one where normal drawing happens, and
  // one phase that is "after" all the other objects have draw. This
  // allows for transparent objects to be drawn last. 
  // The caller will always take care of doing a
  // pushAttributes/matrix, so don't worry about that
  virtual void draw(DrawingState*);
  virtual void drawAfter(DrawingState*);

  // most objects have some way to position themselves
  // by standardizing that, we can make any object "ridable"
  // in general, most objects will have their centers at their
  // local original and use this matrix to move them around
  Matrix transform;

  // if we are "riding" this object (that means that it is the current
  // camera), it has two responsibilities:
  // 1) it must set up the camera transformation (not the projection)
  // 2) it must provide the user interface
  virtual void getCamera(Matrix);	// provide a camera matrix

  virtual bool uiStep();		// called if you're the camera before
								// drawing - lets you do key polling
  virtual int handle(int e);	// exactly what the FlTk Widgets do

  // we will keep a linked list of these things - and I hate STL
  GrObject* next;

  // we also have hierarchy
  GrObject* parent;				// your parent is responsible for you
  GrObject* children;
  GrObject* add(GrObject*, float x=0, float y=0, float z=0, float ry=0);

  // an object might be "interesting" - that is, that we want to put a
  // bookmark in the UI 
  // store the lookat and lookfrom points for these objects - in local
  // coordinates
  bool interesting;
  float laX, laY, laZ;
  float lfX, lfY, lfZ;
};

// add a new object onto the objects list
// allow for positioning in case object doesn't
// note: if you want to make a subobj, use the method of GrObj
extern GrObject* theObjects;
GrObject* add(GrObject*, float x=0, float y=0, float z=0, float ry=0, bool shadow = false);

// draw a list of objects
// note: this has to recurse to draw each objects children!
void drawObList(GrObject* head, DrawingState* drst, bool shadow = false);
void drawAfterObList(GrObject* head, DrawingState* drst);

#endif
