/// GraphicsTown2006 - CS559 Sample Code
// written by Michael Gleicher - originally written Fall, 2000
//
// updated Fall 2006 - more testing code

//
#include "GrTown_PCH.H"
#include "GraphicsTownUI.H"
#include "FlyCamera.H"
#include "GrWorld.H"
#include "DrawUtils.H"
#include "Texture.H"
#include "Examples/Objects.H"
#include "Examples/SimpleBehaviors.H"
#include "Examples/Cars.H"

// Example code (objects and behaviors)
#include "Examples/Suburbs.H"

#include "Roads/Roads.H"
#include "Roads/Drive.H"


#include "MoreObjects.h"
#include "MoreBehaviors.h"
#include "ParticleSystem.h"
#include "shadows.h"

// define this to get 2 cars that always turn
// #define TESTCARS

const int numCars = 100;
const int nGrids = 5;

int main(int /*argc*/, char** /*argv*/)
{ 
	GrObject * sb = new SkyBox;
	add(sb, 0, 0, 0, 0);

	GrObject * shadowSphere = new ShadowSphere(20, 60, 200, 10);
	shadowSpheres.push_back(shadowSphere);
	shadowSphere = new ShadowSphere(30, 60, 200, 10);
	shadowSpheres.push_back(shadowSphere);
	shadowSphere = new ShadowSphere(40, 60, 200, 10);
	shadowSpheres.push_back(shadowSphere);
	add(shadowSphere, 0, 0, 0, 0, false);

	GrObject * bird = new Bird();
	new FlapWings(bird);
	new MoveForward(bird, 3);
	new Turn(bird);
	add(bird, 100, 30, 35, 0, true);

	bird = new Bird();
	new FlapWings(bird);
	new MoveForward(bird, 3);
	new Turn(bird);
	add(bird, 50, 30, 130, 0, true);

//	GrObject * bird = new Bird();
//	add(bird, 0, 0, 10);

	GrObject * particleSystem = new ParticleSystem(1000, 100);
	particleSystem->interesting = true;
	particleSystem->name = "Snow field";
	particleSystem->laX = 400; particleSystem->laY = 0; particleSystem->laZ = 50;
	particleSystem->lfX = 0; particleSystem->lfY = 50; particleSystem->lfZ = 10;
	new SnowStorm(particleSystem);
	add(particleSystem, 0, 0, 330, 0, true);

	GrObject * helicopter = new Helicopter();
	new RotateBlades(helicopter);
	new MoveForward(helicopter);
	new Turn(helicopter);
	add(helicopter, 100, 50, 20, 0, true);

	helicopter = new Helicopter();
	new RotateBlades(helicopter);
	new MoveForward(helicopter);
	new Turn(helicopter);
	add(helicopter, 10, 50, 10, 0, true);

	GrObject * bmHouse = new BumpHouse();
	bmHouse->laX = 0; bmHouse->laY = 0; bmHouse->laZ = 0;
	bmHouse->lfX = -75; bmHouse->lfY = 50; bmHouse->lfZ = 75;
	add(bmHouse, -100, 0, 0, pi/2.f);

  // add some more stuff
  GrObject* o1 = new Church;
  o1->interesting = true;
  o1->laX = 0; o1->laY = 0; o1->laZ = 0;
  o1->lfX = -50; o1->lfY = 100; o1->lfZ = 300;
  add(o1,-100,0,100,pi/2.f);


  // *****************************************************************
  //  Make your town here




  ////////////////////////////////////////////////////////////////////////
  // now to make a real town!
  int r,c;
  // make a 5x5 grid of town blocks - 5 houses per
  for( r=0; r<5; r++) {
	  for( c=0; c<5; c++) {
		  if (c == 2 && r == 0) {
			  SubDivision2 * sd = new SubDivision2();
			  sd->interesting = true;
			  sd->name = "Ogro field";
		      sd->laX = 400; sd->laY = 0; sd->laZ = 100;
			  sd->lfX = 150; sd->lfY = 50; sd->lfZ = 10;
			  new OgrosMove(sd);
			  add(sd, static_cast<float>(r*530),0,static_cast<float>(c*230));
		  } else add(new SimpleSubdivision(5),static_cast<float>(r*530),0,static_cast<float>(c*230));
		  add(new StraightRoad(static_cast<float>(r*530),static_cast<float>(c*230),static_cast<float>(r*530+500),static_cast<float>(c*230)));
	  }
  }

  // make cross streets
  for(int r=0; r<=5; r++) {
	  for(c=0; c<4; c++) {
		  add(new StraightRoad(static_cast<float>(r*530 - 15), static_cast<float>(c*230 + 15), static_cast<float>(r*530 - 15), static_cast<float>(c*230+215)));
	  }
  }

  // make intersections
  // make an intersection intersesting so we can look at it
  for(int r=0; r<=5; r++) {
	  for(c=0; c<5; c++) {
		  GrObject* g = new Intersection(static_cast<float>(r*530-15), static_cast<float>(c*230));
		  if ( (r==2) && (c==3) ) {
			  g->interesting = 1;
			  g->name = "Intersection(2,3)";
			  g->laX = static_cast<float>(r*530-15);    g->laY = 0;    g->laZ = static_cast<float>(c*230);
			  g->lfX = static_cast<float>(r*530+25);   g->lfY = 100;   g->lfZ = static_cast<float>(c*230+150);
		  }
		  add(g);
	  }
  }

#ifndef TESTCARS
  // add some cars
  for(int r=0; r<50; r++) {
	Car* c;
	switch(rand() % 3) {
	  case 0: c = new Van(rand()); break;
	  case 1: c = new SUV(rand()); break;
	  case 2: c = new HatchBack(rand()); break;
	}
	add(c);
    new RandomDrive(c,theRoads[rand() % theRoads.size()],.2f,rand() % 2);
  }
#endif

#ifdef TESTCARS
  // two cars - one always turns right, one always turns left
  Car* c1 = new HatchBack(1);
  Car* c2 = new HatchBack(2);
	add(c1);
	add(c2);
  new RandomDrive(c1,theRoads[7],.5,0,-1);
  new RandomDrive(c2,theRoads[8],.5,0,1);

#endif


	// a race track
    Road* t = new RoundRoad(-250,250,100);
	add(t);
	t->name = "Track";
	t->interesting = true;
	t->laX = -250; t->laY = 0;   t->laZ = 250;
	t->lfX = -600; t->lfY = 200; t->lfZ = 450;
	// make cars go around the track
	Car* h = new HatchBack(1);
	h->name="Race1";		// warning! we can only do this since we don't delete
	add(h);
	new SimpleDrive(h,t,0,0);
	h = new HatchBack(3);
	h->name="Race2";		// warning! we can only do this since we don't delete
	add(h);
	Drive* d = new SimpleDrive(h,t,0,1);
	d->speed *= 2;


  // *****************************************************************
  // now make a UI
  FlyCamera* fc = new FlyCamera;
  Map* m = new Map;

  add(fc);
  add(m);

  GraphicsTownUI grTown;
  grTown.make_window();
  grTown.townView->defaultCamera = fc;
  grTown.window->show();

  Fl::run();
  return 1;
}

