//  MoreBehaviors.cpp
//  Description: More behaviors for objects

#include "MoreBehaviors.h"
#include <iostream>

using namespace std;

RotateBlades::RotateBlades(GrObject * o, float sp) : Behavior(o), speed(sp)
{
}

void RotateBlades::simulateUntil(unsigned long time)
{
	Helicopter * hc = (Helicopter *)owner;
	
	hc->setRotFactor((float)(time - lastV) / 2.f);
	lastV = time;
}

FlapWings::FlapWings(GrObject * o, float sp) : Behavior(o), speed(sp)
{
}

void FlapWings::simulateUntil(unsigned long time)
{
	Bird * b = (Bird *)owner;

	b->update((float)(time-lastV) * speed);

	lastV = time;
}

MoveForward::MoveForward(GrObject * o, float sp) : Behavior(o), speed(sp)
{
}

void MoveForward::simulateUntil(unsigned long time)
{
	float amt = speed * (float)(time - lastV) / 100;
	float x = owner->transform[0][2] * amt;
	float y = owner->transform[1][2] * amt;
	float z = owner->transform[2][2] * amt;

	owner->transform[3][0] += x;
	owner->transform[3][1] += y;
	owner->transform[3][2] += z;

	lastV = time;
}

Turn::Turn(GrObject* o) : Behavior(o)
{
}

void Turn::simulateUntil(unsigned long time)
{
	float x = owner->transform[3][0];
	float y = owner->transform[3][1];
	float z = owner->transform[3][2];

	if ((rand() % 100) == 0) {
		// set rotation based on time
		rotMatrix(owner->transform, 'y', (float)(rand() % 45));

		// put the position back
		owner->transform[3][0] = x;
		owner->transform[3][1] = y;
		owner->transform[3][2] = z;
	}

	lastV = time;
}

OgrosMove::OgrosMove(GrObject* o) : Behavior(o)
{
}

void OgrosMove::simulateUntil(unsigned long time)
{
	SubDivision2 * sd = (SubDivision2 *)owner;

	sd->updateOgroField(((float)time - lastV) / 100.f);

	lastV = time;
}

SnowStorm::SnowStorm(GrObject* o) : Behavior(o)
{
}

void SnowStorm::simulateUntil(unsigned long time)
{
	ParticleSystem * storm = (ParticleSystem *)owner;

	storm->update(time - lastV);

	lastV = time;
}