//  MoreObjects.cpp
//  Description: More objects

#include "MoreObjects.h"
#include <iostream>
#include <math.h>
#include "DrawingState.h"
#include "Texture.h"
#include "MMatrix.h"
#include "Primitives.h"
#include "MoreBehaviors.h"

using namespace std;

static MD2Model * ogroModel = NULL;

Helicopter::Helicopter()
{
	degree = 0;
	rotMatrix(transform, 'y', 90);
}

void Helicopter::draw(DrawingState * d)
{
	glPushMatrix();
		if (!show_shadow) glColor3f(.7f, 0, 0);
		//glScalef(3, 1, 1);
		primitives_DrawCube(!show_shadow);
	glPopMatrix();

	glPushMatrix();
		glRotatef(degree, 0, 1, 0);
		glPushMatrix();
			if (!show_shadow) glColor3f(.7f, .7f, .7f);
			glTranslatef(0, 1.5, 0);
			glScalef(.2f, .75, .2f);
			primitives_DrawCube(false);
		glPopMatrix();

		glTranslatef(0, 2, 0);
		glPushMatrix();
			glScalef(5, 0.03f, 0.1f);
			primitives_DrawCube(false);
		glPopMatrix();

		glPushMatrix();
			glScalef(0.1f, 0.03f, 5);
			primitives_DrawCube(false);
		glPopMatrix();
	glPopMatrix();
}

void Helicopter::setRotFactor(float deg)
{
	degree += deg;
	if (degree > 360)
		degree = (float)((int)degree % 360);
}

SubDivision2::SubDivision2()
{
	// Two areas +- 20, 50 - 450, 100
	add(new TennisCourt(), 20, 0, 50);
	add(new Church2(), 150, 0, 70);
	add((field = new OgroField()), 200, 0, 50);
}

void SubDivision2::draw(DrawingState * d)
{
}

void SubDivision2::updateOgroField(float time)
{
	field->update(time);
}


TennisCourt::TennisCourt()
{
}

void TennisCourt::draw(DrawingState * d)
{
	static unsigned int courtTex = fetchTexture("tennis_court.bmp", true, true);

	glBindTexture(GL_TEXTURE_2D, courtTex);

	glBegin(GL_QUADS);
		glColor3f(1, 1, 1);
		glNormal3f(0, 1, 0);
		glTexCoord2i(0, 0); glVertex3f(0, 1.0f, 0);
		glTexCoord2i(1, 0); glVertex3f(0, 1.0f, 50);
		glTexCoord2i(1, 1); glVertex3f(100, 1.0f, 50);
		glTexCoord2i(0, 1); glVertex3f(100, 1.0f, 0);
	glEnd();
	glBindTexture(GL_TEXTURE_2D, 0);

	glPushMatrix();
		glTranslatef(10, 0, 47);
		primitives_lamp();
		glTranslatef(25, 0, 0);
		primitives_lamp();
		glTranslatef(25, 0, 0);
		primitives_lamp();
		glTranslatef(25, 0, 0);
		primitives_lamp();
	glPopMatrix();	
}


Church2::Church2()
{
	md2 = NULL;
}

Church2::~Church2()
{
	if (md2 != NULL)
		delete md2;
}

void Church2::draw(DrawingState * d)
{
	if (md2 == NULL)
		md2 = new MD2Model("Models/q_church.md2", "");

	glPushMatrix();
		glRotatef(-90, 1, 0, 0);
		glColor3f(1, 1, .8f);
		md2->render(0);
	glPopMatrix();
}

Ogro::Ogro(float x, float z)
{
	posX = x;
	posZ = z;
	rot = 0;
	state = 0;
	frame = 0;
	interpolate = 0;
}

Ogro::~Ogro()
{
}

void Ogro::draw(DrawingState * d)
{
	int next = frame+1;

	if (ogroModel == NULL)
		ogroModel = new MD2Model("Models/Ogro.md2", "Models/Ogro.bmp");

	if (state == 0 && frame == 39) {
		next = 0;
	} else if (state == 1 && frame == 45) {
		next = 40;
	}

	glPushMatrix();
		glTranslatef(posX, 0, posZ);
		glScalef(.3f, .3f, .3f);
		glRotatef(-90, 1, 0, 0);
		glTranslatef(0, 0, 25);
		glRotatef(rot, 0, 0, 1);

		if ((d->timeOfDay < 7) || (d->timeOfDay > 18)) {
			state = 2;
			ogroModel->render(197);
		} else {
			if (state == 2) state = 0;
			ogroModel->animate(frame, next, interpolate);
		}
	glPopMatrix();
}

void Ogro::changeState(void)
{
	if (state == 0) {
		state = 1;
		frame = 40;
	} else if (state == 1) {
		state = 0;
		frame = 0;
	}
}

void Ogro::updateTime(float time)
{
	float moreFrames = time + interpolate;

	if (state == 1) {
		posX += cos(rot * 3.1415962f / 180.0f);
		posZ -= sin(rot * 3.1415962f / 180.0f);
	}

	while (moreFrames > 1) {
		this->nextFrame();
		moreFrames--;
	}

	interpolate = moreFrames;

	if (posX > 250) {
		posX = 250;
		state = 0;
		rot += 90;
	} if (posZ > 70) {
		posZ = 70;
		state = 0;
		rot += 90;
	} if (posX < 0) {
		posX = 0;
		state = 0;
		rot += 90;
	} if (posZ < 0) {
		posZ = 0;
		state = 0;
		rot += 90;
	}
}

void Ogro::nextFrame(void)
{
	frame++;
	if (state == 0 && frame == 40) {
		frame = 0;
	} else if (state == 1 && frame == 46) {
		frame = 40;
	}
}

void Ogro::rotate(float r)
{
	rot += r;
	if (rot < 0)
		rot = -rot + 180;
	if (rot > 360)
		rot = (float)((int)rot % 360);
}

OgroField::OgroField()
{
	Ogro * og = new Ogro(0, 0);
	new OgrosMove(og);
	add(og, 0, 0, 0);
	og = new Ogro(20, 30);
	new OgrosMove(og);
	add(og, 0, 0, 0);
	og = new Ogro(40, 10);
	new OgrosMove(og);
	add(og, 0, 0, 0);
}

void OgroField::draw(DrawingState * d)
{
}

void OgroField::update(float time)
{
	Ogro * og = (Ogro *)this->children;

	while (og != NULL) {
		if (og->state < 2) {
			if (rand() % 50 == 0)
				og->changeState();
			if (og->state == 0) {
				if (rand() % 50 == 0)
					og->rotate((float)(rand() % 100));
			}
			og->updateTime(time);
		}
		og = (Ogro *)og->next;
	}
}

Bird::Bird()
{
	rotAmt = 0;
	moveUp = true;
}

void Bird::draw(DrawingState *)
{
	static GLUquadricObj * ob = gluNewQuadric();
	GLboolean culls = glIsEnabled(GL_CULL_FACE);

	glDisable(GL_CULL_FACE);
	if (!show_shadow) glColor3f(.8f, .8f, .8f);
	glPushMatrix();
		//glRotatef(90, 0, 1, 0);
		glScalef(1, 1, 5);
		gluSphere(ob, 1, 4, 4);
		glScalef(1, 1, .1f);

		glRotatef(rotAmt, 0, 0, 1);
		glBegin(GL_TRIANGLES);
			if (!show_shadow) glColor3f(1, 1, .7f);
			glNormal3f(0, 1, 0);
			glVertex3f(-5, 0, 0);
			glVertex3f(0, 0, 3);
			glVertex3f(0, 0, -3);

			glNormal3f(0, 0, 1);
			glVertex3f(-5, 0, 0);
			glVertex3f(0, 1, 0);
			glVertex3f(0, -1, 0);
		glEnd();

		glRotatef(-2 * rotAmt, 0, 0, 1);
		glBegin(GL_TRIANGLES);
			glNormal3f(0, 1, 0);
			glVertex3f(5, 0, 0);
			glVertex3f(0, 0, 3);
			glVertex3f(0, 0, -3);

			glNormal3f(0, 0, 1);
			glVertex3f(5, 0, 0);
			glVertex3f(0, 1, 0);
			glVertex3f(0, -1, 0);
		glEnd();
	glPopMatrix();
	if (culls) glEnable(GL_CULL_FACE);
}

void Bird::update(float time)
{
	time /= 10.f;
	rotAmt += (moveUp ? time : -time);
	if (rotAmt > 20) {
		rotAmt = 20;
		moveUp = false;
	} else if (rotAmt < -20){
		rotAmt = -20;
		moveUp = true;
	}
}