//  Steven Kappes
//  Primitives.cpp
//  Description: Functions to draw primitive objects

#include "Primitives.h"
#include "Texture.h"

// OpenGL includes
#include <Fl/Gl.h>
#include <Gl/Glu.h>

void primitives_DrawCube(bool perspectiveLines)
{
	glBegin(GL_QUADS);
		// front
		glNormal3f(0, 0, 1);
		glVertex3f(-1, 1, 1);
		glVertex3f(-1, -1, 1);
		glVertex3f(1, -1, 1);
		glVertex3f(1, 1, 1);

		// back
		glNormal3f(0, 0, -1);
		glVertex3f(-1, 1, -1);
		glVertex3f(1, 1, -1);
		glVertex3f(1, -1, -1);
		glVertex3f(-1, -1, -1);

		// top
		glNormal3f(0, 1, 0);
		glVertex3f(-1, 1, -1);
		glVertex3f(-1, 1, 1);
		glVertex3f(1, 1, 1);
		glVertex3f(1, 1, -1);

		// bottom
		glNormal3f(0, -1, 0);
		glVertex3f(-1, -1, -1);
		glVertex3f(1, -1, -1);
		glVertex3f(1, -1, 1);
		glVertex3f(-1, -1, 1);

		// left
		glNormal3f(-1, 0, 0);
		glVertex3f(-1, 1, -1);
		glVertex3f(-1, -1, -1);
		glVertex3f(-1, -1, 1);
		glVertex3f(-1, 1, 1);

		// right
		glNormal3f(1, 0, 0);
		glVertex3f(1, 1, 1);
		glVertex3f(1, -1, 1);
		glVertex3f(1, -1, -1);
		glVertex3f(1, 1, -1);
	glEnd();

	if (perspectiveLines) {
		glLineWidth(1.5f);
		glBegin(GL_LINES); // Draw the edges of the cube for perspective
			glColor3f(0, 0, 0); // Draw edges in black
			glVertex3f(-1, -1, -1);
			glVertex3f(-1, -1, 1);

			glVertex3f(-1, -1, -1);
			glVertex3f(-1, 1, -1);

			glVertex3f(-1, -1, -1);
			glVertex3f(1, -1, -1);

			glVertex3f(1, -1, -1);
			glVertex3f(1, -1, 1);

			glVertex3f(1, -1, -1);
			glVertex3f(1, 1, -1);

			glVertex3f(-1, 1, -1);
			glVertex3f(1, 1, -1);

			glVertex3f(-1, 1, -1);
			glVertex3f(-1, 1, 1);

			glVertex3f(-1, -1, 1);
			glVertex3f(-1, 1, 1);

			glVertex3f(-1, -1, 1);
			glVertex3f(1, -1, 1);

			glVertex3f(-1, 1, 1);
			glVertex3f(1, 1, 1);

			glVertex3f(1, -1, 1);
			glVertex3f(1, 1, 1);

			glVertex3f(1, 1, -1);
			glVertex3f(1, 1, 1);
		glEnd();
		glLineWidth(1);
	}
}

void primitives_brickWall(float width, float height, bool bump)
{
	// Draw a brick wall
	float brickHeight = 2.5f;
	float brickWidth = 5.0f;
	float cementSize = .3f;
	float brickHeightSize = brickHeight - cementSize;
	float brickWidthSize = brickWidth - cementSize;
	float i, j;
	static unsigned int brick1ID = fetchTexture("brick1.bmp", true, true);
	static unsigned int brick2ID = fetchTexture("brick2.bmp", true, true);
	bool offset = false;
	bool brick1 = true;

	for (j = 0; j < height; j += brickHeight) {
		for (i = 0; i < width; i += brickWidth) {
			if ((i == 0) && (offset)) { // Draw first brick smaller
				brickWidth = brickWidth / 2;
				brickWidthSize = brickWidth - cementSize;cementSize;
			} else if ((i == brickWidth) && (offset)) { // Draw regular sized bricks
				brickWidth = brickWidth * 2;
				brickWidthSize = brickWidth - cementSize;
			} else if ((offset) && (i + brickWidth > width)) { // Draw last brick smaller
				brickWidth = brickWidth / 2;
				brickWidthSize = brickWidth - cementSize;
			}

			// Draw a brick
			glBindTexture(GL_TEXTURE_2D, (brick1 ? brick1ID : brick2ID));
			brick1 = !brick1;
			glBegin(GL_QUADS);
				glColor3f(1, 1, 1);
				glNormal3f(0, 0, 1);
				glTexCoord2i(0, 0);
				glVertex3f(i, j, 0);

				glNormal3f(0, 0, 1);
				glTexCoord2i(1, 0);
				glVertex3f(i + brickWidthSize, j, 0);

				glNormal3f(0, 0, 1);
				glTexCoord2i(1, 1);
				glVertex3f(i + brickWidthSize, j + brickHeightSize, 0);

				glNormal3f(0, 0, 1);
				glTexCoord2i(0, 1);
				glVertex3f(i, j + brickHeightSize, 0);
			glEnd();

			// Draw cement
			glBindTexture(GL_TEXTURE_2D, 0);
			glBegin(GL_QUADS);
				glColor3f(.5f, .5f, .5f);
				if (bump) glNormal3f(1, 0, .1f);
				else glNormal3f(0, 0, 1);
				glTexCoord2d(0, 0);
				glVertex3f(i + brickWidthSize, j, 0);

				if (bump) glNormal3f(-1, 0, .1f);
				else glNormal3f(0, 0, 1);
				glTexCoord2d(.1, 0);
				glVertex3f(i + brickWidth, j, 0);

				if (bump) glNormal3f(-1, 0, .1f);
				else glNormal3f(0, 0, 1);
				glTexCoord2d(.1, .3);
				glVertex3f(i + brickWidth, j + brickHeightSize, 0);

				if (bump) glNormal3f(1, 0, .1f);
				else glNormal3f(0, 0, 1);
				glTexCoord2d(0, .3);
				glVertex3f(i + brickWidthSize, j + brickHeightSize, 0);				
			glEnd();
		}

		// Draw cement
		glBegin(GL_QUADS);
			glColor3f(.5f, .5f, .5f);
			if (bump) glNormal3f(0, 1, .1f);
			else glNormal3f(0, 0, 1);
			glTexCoord2d(0, 0);
			glVertex3f(0, j + brickHeightSize, 0);

			if (bump) glNormal3f(0, 1, .1f);
			else glNormal3f(0, 0, 1);
			glTexCoord2d(10, 0);
			glVertex3f(width, j + brickHeightSize, 0);

			if (bump) glNormal3f(0, -1, .1f);
			else glNormal3f(0, 0, 1);
			glTexCoord2d(10, .3);
			glVertex3f(width, j + brickHeight, 0);

			if (bump) glNormal3f(0, -1, .1f);
			else glNormal3f(0, 0, 1);
			glTexCoord2d(0, .3);
			glVertex3f(0, j + brickHeight, 0);
		glEnd();

		offset = !offset;
		if (!offset) { // Draw regular sized bricks
			brickWidth = brickWidth * 2;
			brickWidthSize = brickWidth - cementSize;
		}
	}
	glBindTexture(GL_TEXTURE_2D, 0);
}

void primitives_lamp(void)
{
	double poleZ = 9;

	glColor3f(.6f,.5f,.3f);
	glBegin(GL_QUAD_STRIP);
		glNormal3d( 1, 0,0);
		glVertex3d( 1, 0,poleZ);
		glVertex3d( 1,20,poleZ);
		glNormal3d( 0, 0,1);
		glVertex3d( 0, 0,poleZ+1);
		glVertex3d( 0,20,poleZ+1);
		glNormal3d(-1, 0,0);
		glVertex3d(-1, 0,poleZ);
		glVertex3d(-1,20,poleZ);
		glNormal3d( 0, 0,-1);
		glVertex3d( 0, 0,poleZ-1);
		glVertex3d( 0,20,poleZ-1);
		glNormal3d( 1, 0,0);
		glVertex3d( 1, 0,poleZ);
		glVertex3d( 1,20,poleZ);
	glEnd();
	glBegin(GL_QUAD_STRIP);
		glNormal3d(0,1,0);
		glVertex3d( 0,21, 0);
		glVertex3d( 0,21, poleZ);
		glNormal3d(-.707,-.707, 0);
		glVertex3d(-1,20, 0);
		glVertex3d(-1,20, poleZ);
		glNormal3d(.707,-.707, 0);
		glVertex3d( 1,20, 0);
		glVertex3d( 1,20, poleZ);
		glNormal3d(0,1,0);
		glVertex3d( 0,21, 0);
		glVertex3d( 0,21, poleZ);
	glEnd();
}
