#include "shadows.h"

std::vector<GrObject *> shadowSpheres;

void ShadowSphere::draw(DrawingState * d)
{
	ShadowSphere * sp;
	unsigned int i;
	for (i = 0; i < shadowSpheres.size(); i++) {
		sp = (ShadowSphere *)shadowSpheres.at(i);	
		glPushMatrix();
			glTranslatef(sp->x, sp->y, sp->z);
			glColor3f(sp->rc, sp->gc, sp->bc);
			DrawSphere(sp->r);
		glPopMatrix();
	}
}

void ShadowSphere::drawShadowVolume(float lightPos[4], float infinity, float light_r){
	glPushMatrix();
		glRotatef(90, 0, 1, 0);
		//calculate shadow angle
		float dx = lightPos[0] - x;
		float dy = lightPos[1] - y;
		float dz = lightPos[2] - z;
		float dist = sqrt(dx*dx + dy*dy + dz*dz);

		float rx = (float)radToDeg(atan2(dy, dz));
		float ry = (float)radToDeg(atan2(dz, dx));
		float rz = (float)radToDeg(atan2(dy, dx));

		//orient cylindar
		glRotatef(rx, 1, 0, 0);
		glRotatef(ry, 0, 1, 0);
		glRotatef(rz, 0, 0, 1);

		glTranslatef(infinity, 0, 0);
		glRotatef(-90, 0, 1, 0);

		//calulate base radius
		float sr, br;
		sr = r;
		br = r *(infinity) / dist;
		
		//draw shadow cylinder
		glRotatef(180, 0, 1, 0);
		DrawDisk(br);					//draw base cap
		glRotatef(-180, 0, 1, 0);
		DrawCylinder(br, sr, infinity);	//draw cylindar
		glTranslatef(0, 0, infinity);
		DrawDisk(sr);					//draw top cap
	glPopMatrix();
}

void DrawDisk(float r){
	GLUquadricObj *disk;
	disk = gluNewQuadric();
	gluQuadricDrawStyle(disk, GLU_FILL);
	gluQuadricNormals(disk, GLU_SMOOTH);
	//glColor4f(0, 0, 0, 0);
	gluDisk(disk, 0, r, 20, 20);
	gluDeleteQuadric(disk);
}

void DrawSphere(float r){
	GLUquadricObj *sp;
	sp = gluNewQuadric();
	gluQuadricDrawStyle(sp, GLU_FILL);
	gluQuadricNormals(sp, GLU_SMOOTH);
	
	gluSphere(sp, r, 20, 20);
	gluDeleteQuadric(sp);
}

void DrawCylinder(float br, float tr, float h){
	GLUquadricObj *cyl;
	cyl = gluNewQuadric();
	gluQuadricDrawStyle(cyl, GLU_FILL);
	gluQuadricNormals(cyl, GLU_SMOOTH);
	gluCylinder(cyl, br, tr, h, 20, 20);
	gluDeleteQuadric(cyl);
}

void drawShadowVolumes(float * lPos){
	float infinity = 10000;
	for(unsigned int i = 0; i < shadowSpheres.size(); i++){
		glPushMatrix();
		glTranslatef(((ShadowSphere*)shadowSpheres.at(i))->x, ((ShadowSphere*)shadowSpheres.at(i))->y, ((ShadowSphere*)shadowSpheres.at(i))->z);
		((ShadowSphere *)(shadowSpheres.at(i)))->drawShadowVolume(lPos, infinity);
		glPopMatrix();
	}
}

void drawVolumetricShadows(float *lPos){
	//printf("lPos: %f, %f, %f, %f\n", lPos[0], lPos[1], lPos[2], lPos[4]);
	float llPos[4];
	
	llPos[0] = lPos[0] * 100;  
	llPos[1] = lPos[1] * 100;  
	llPos[2] = 0;  
	llPos[3] = 0;  
	lPos[0] = lPos[0] * 100;  
	lPos[1] = lPos[1] * 100;  
	lPos[2] = 0;  
	lPos[3] = 0; 

	float infinity = 10000;
	for(unsigned int i = 0; i < shadowSpheres.size(); i++) {
		glPushMatrix();
			glTranslatef(((ShadowSphere*)shadowSpheres.at(i))->x, ((ShadowSphere*)shadowSpheres.at(i))->y, ((ShadowSphere*)shadowSpheres.at(i))->z);
			glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
			glDepthMask(GL_FALSE);
			glEnable(GL_CULL_FACE);
			glEnable(GL_STENCIL_TEST);
			glEnable(GL_POLYGON_OFFSET_FILL);
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA); 
			glPolygonOffset(0.0f, 1.0f);

			glCullFace(GL_FRONT);
			glStencilFunc(GL_ALWAYS, 0x0, 0xff);
			glStencilOp(GL_KEEP, GL_INCR, GL_KEEP);
			((ShadowSphere *)(shadowSpheres.at(i)))->drawShadowVolume(llPos, infinity);

			glCullFace(GL_BACK);
			glStencilFunc(GL_ALWAYS, 0x0, 0xff);
			glStencilOp(GL_KEEP, GL_DECR, GL_KEEP);
			((ShadowSphere *)(shadowSpheres.at(i)))->drawShadowVolume(llPos, infinity);

			glDisable(GL_POLYGON_OFFSET_FILL);
			glDisable(GL_CULL_FACE);
			glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
			glDepthMask(GL_TRUE);

			glStencilFunc(GL_NOTEQUAL, 0x0, 0xff);
			glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);
			drawShadow();
			glDisable(GL_STENCIL_TEST);
		glPopMatrix();
	}
}

void drawShadow(){
	glPushMatrix();
	glLoadIdentity();
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho(0, 1, 1, 0, 0, 1);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glColor4f(0.0f, 0.0f, 0.0f, 0.5f);
	glBegin(GL_QUADS);
		glVertex2i(0, 0);
		glVertex2i(0, 1);
		glVertex2i(1, 1);
		glVertex2i(1, 0);
	glEnd();
	glDisable(GL_BLEND);
	glEnable(GL_DEPTH_TEST);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}

void setShadowMapMatrix(GLfloat dest[16], float lightPos[4], float plane[4]){
	/*
	printf("light:(%f,%f,%f,%f) plane:(%f,%f,%f,%f)",
			lightPos[0], lightPos[1], lightPos[2], lightPos[3], 
			plane[0], plane[1], plane[2], plane[3]);
	*/
	GLfloat dot;
	dot = lightPos[0] * plane[0] + lightPos[1] * plane[1] +
		  lightPos[2] * plane[2] + lightPos[3] * plane[3];
	
	//first column
	dest[0]  = dot  - lightPos[0] * plane[0];
	dest[4]  = 0.0f - lightPos[0] * plane[1];
	dest[8]  = 0.0f - lightPos[0] * plane[2];
	dest[12] = 0.0f - lightPos[0] * plane[3];
	
	//second column
	dest[1]  = 0.0f - lightPos[1] * plane[0];
	dest[5]  = dot  - lightPos[1] * plane[1];
	dest[9]  = 0.0f - lightPos[1] * plane[2];
	dest[13] = 0.0f - lightPos[1] * plane[3];

	//third column
	dest[2]  = 0.0f - lightPos[2] * plane[0];
	dest[6]  = 0.0f - lightPos[2] * plane[1];
	dest[10] = dot  - lightPos[2] * plane[2];
	dest[14] = 0.0f - lightPos[2] * plane[3];

	//fourth column
	dest[3]  = 0.0f - lightPos[3] * plane[0];
	dest[7]  = 0.0f - lightPos[3] * plane[1];
	dest[11] = 0.0f - lightPos[3] * plane[2];
	dest[15] = dot  - lightPos[3] * plane[3];
}

void drawGroundPlane(float plane[4]){
	float r = 1000;
	glColor3f(.2f, .2f, .5f);
	glBegin(GL_QUADS);
		glVertex3f(-r, -plane[3], r);
		glVertex3f(r, -plane[3], r);
		glVertex3f(r, -plane[3], -r);
		glVertex3f(-r, -plane[3], -r);
	glEnd();
}

void drawMappedShadows(float lightPos[4], float plane[4], DrawingState* drst){
	//gluLookAt
	//clear the screen
	//glClearStencil(0);
	//glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
	/*
	//turn off other buffers
	glColorMask(GL_FALSE, GL_FALSE, GL_FALSE, GL_FALSE);
	glDepthMask(GL_FALSE);

	//set up stencil
	glEnable(GL_STENCIL_TEST);
	glStencilFunc(GL_ALWAYS, 1, 0xFFFFFFFF);
	glStencilOp(GL_REPLACE, GL_REPLACE, GL_REPLACE);

	//draw plane
	drawGroundPlane(plane);

	//turn other buffers back on
	glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
	glDepthMask(GL_TRUE);

	glStencilFunc(GL_EQUAL, 1, 0xFFFFFFFF);
	glStencilOp(GL_KEEP, GL_KEEP, GL_KEEP);

	drawGroundPlane(plane);
	*/
	//draw shadow
	glPushMatrix();
		GLfloat shadowMatrix[16];
		setShadowMapMatrix(shadowMatrix, lightPos, plane);
		
		/*
		for(int i = 0; i < 16; i++){
			cout << shadowMatrix[i] << " ";
			if((i+1)%4 == 0) cout << endl;
		}
		cout << endl;
		*/

		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		//glDisable(GL_DEPTH_TEST);
		glEnable(GL_BLEND);

		//glStencilOp(GL_KEEP, GL_KEEP, GL_INCR);
		//glColor4f(0.0f, 0.0f, 0.0f, 0.5f);

		//project through matrix
		glMultMatrixf(shadowMatrix);
		drawObList(theObjects, drst, true);

		glEnable(GL_TEXTURE_2D);
		//glEnable(GL_DEPTH_TEST);
		glDisable(GL_BLEND);
		glEnable(GL_LIGHTING);

	glPopMatrix();
	//glDisable(GL_STENCIL_TEST);
	
	//drawGroundPlane(plane);
	/*
	glPushMatrix();
	glColor3f(1, 1, 1);
	drawPlanets();
	glPopMatrix();
	*/
}